defmodule DecentralisedBookIndexWeb.DbiServerLive.Index do
  use DecentralisedBookIndexWeb, :live_view

  @impl true
  def render(assigns) do
    ~H"""
    <.header>
      Listing servers
      <:actions>
        <.link patch={~p"/servers/new"}>
          <.button>New server</.button>
        </.link>
      </:actions>
    </.header>

    <.table
      id="dbi_servers"
      rows={@streams.dbi_servers}
      row_click={fn {_id, dbi_server} -> JS.navigate(~p"/servers/#{dbi_server}") end}
    >
      <:col :let={{_id, dbi_server}} label="Name">{dbi_server.name}</:col>

      <:col :let={{_id, dbi_server}} label="Url">{dbi_server.url}</:col>

      <:action :let={{_id, dbi_server}}>
        <div class="sr-only">
          <.link navigate={~p"/servers/#{dbi_server}"}>Show</.link>
        </div>

        <.link patch={~p"/servers/#{dbi_server}/edit"}>Edit</.link>
      </:action>

      <:action :let={{id, dbi_server}}>
        <.link
          phx-click={JS.push("delete", value: %{id: dbi_server.id}) |> hide("##{id}")}
          data-confirm="Are you sure?"
        >
          Delete
        </.link>
      </:action>
    </.table>

    <.modal
      :if={@live_action in [:new, :edit]}
      id="dbi_server-modal"
      show
      on_cancel={JS.patch(~p"/servers")}
    >
      <.live_component
        module={DecentralisedBookIndexWeb.DbiServerLive.FormComponent}
        id={(@dbi_server && @dbi_server.id) || :new}
        title={@page_title}
        current_user={@current_user}
        action={@live_action}
        dbi_server={@dbi_server}
        patch={~p"/servers"}
      />
    </.modal>
    """
  end

  @impl true
  def mount(_params, _session, socket) do
    {:ok,
     socket
     |> stream(
       :dbi_servers,
       Ash.read!(DecentralisedBookIndex.Metadata.DBIServer, actor: socket.assigns[:current_user])
     )
     |> assign_new(:current_user, fn -> nil end)}
  end

  @impl true
  def handle_params(params, _url, socket) do
    {:noreply, apply_action(socket, socket.assigns.live_action, params)}
  end

  defp apply_action(socket, :edit, %{"id" => id}) do
    socket
    |> assign(:page_title, "Edit server")
    |> assign(
      :dbi_server,
      Ash.get!(DecentralisedBookIndex.Metadata.DBIServer, id, actor: socket.assigns.current_user)
    )
  end

  defp apply_action(socket, :new, _params) do
    socket
    |> assign(:page_title, "New server")
    |> assign(:dbi_server, nil)
  end

  defp apply_action(socket, :index, _params) do
    socket
    |> assign(:page_title, "Listing servers")
    |> assign(:dbi_server, nil)
  end

  @impl true
  def handle_info(
        {DecentralisedBookIndexWeb.DbiServerLive.FormComponent, {:saved, dbi_server}},
        socket
      ) do
    {:noreply, stream_insert(socket, :dbi_servers, dbi_server)}
  end

  @impl true
  def handle_event("delete", %{"id" => id}, socket) do
    dbi_server =
      Ash.get!(DecentralisedBookIndex.Metadata.DBIServer, id, actor: socket.assigns.current_user)

    Ash.destroy!(dbi_server, actor: socket.assigns.current_user)

    {:noreply, stream_delete(socket, :dbi_servers, dbi_server)}
  end
end
